<?php

namespace Intergo\Wallet\ValueObjects;

use Intergo\Wallet\Enums\BalanceEnum;

class TransactionInfoVO
{
    private $user_id;
    private float $amount = 0.0;
    private string $note = '';
    private string $reason;
    private string $remarks = '';
    private float $balance = 0.0;
    private float $escrowBalance = 0.0;
    private float $new_balance = 0.0;
    private int $count_transactions = 0;

    /**
     * @param $userId
     * @param $amount
     * @param array $information
     */
    public function __construct($userId, $amount, array $information = [])
    {
        $this->setUserId($userId);
        $this->setAmount($amount);
        $this->setNote($information['note'] ?? '');
        $this->setReason($information['reason'] ?? null);
        $this->setRemarks(json_encode($information));
    }

    /**
     * @return int|string
     */
    public function getUserId()
    {
        return $this->user_id;
    }

    /**
     * @param $user_id
     */
    public function setUserId($user_id): void
    {
        $this->user_id = $user_id;
    }

    /**
     * @return float
     */
    public function getAmount(): float
    {
        return $this->amount;
    }

    /**
     * @param float $amount
     */
    public function setAmount(float $amount): void
    {
        $this->amount = $amount;
    }

    /**
     * @return string
     */
    public function getNote(): string
    {
        return $this->note;
    }

    /**
     * @param string $note
     */
    public function setNote(string $note): void
    {
        $this->note = $note;
    }

    /**
     * @return ?string
     */
    public function getReason(): ?string
    {
        return $this->reason;
    }

    /**
     * @param ?string $reason
     */
    public function setReason(?string $reason): void
    {
        if (empty($reason)) {
            $reason = BalanceEnum::DEFAULT_REASON;
        }
        $this->reason = $reason;
    }

    /**
     * @return string
     */
    public function getRemarks(): string
    {
        return $this->remarks;
    }

    /**
     * @param string $remarks
     */
    public function setRemarks(string $remarks): void
    {
        $this->remarks = $remarks;
    }

    /**
     * @return float
     */
    public function getBalance(): float
    {
        return $this->balance;
    }

    /**
     * @param float $balance
     * @return float
     */
    public function setBalance(float $balance): float
    {
        return $this->balance = $balance;
    }

    /**
     * @return float
     */
    public function getEscrowBalance(): float
    {
        return $this->escrowBalance;
    }

    /**
     * @param float $escrowBalance
     * @return float
     */
    public function setEscrowBalance(float $escrowBalance): float
    {
        return $this->escrowBalance = $escrowBalance;
    }

    /**
     * @return float
     */
    public function getNewBalance(): float
    {
        return $this->new_balance;
    }

    /**
     * @param float $new_balance
     * @return float
     */
    public function setNewBalance(float $new_balance): float
    {
        return $this->new_balance = $new_balance;
    }

    /**
     * @return int
     */
    public function getCountTransactions(): int
    {
        return $this->count_transactions;
    }
    /**
     * @param int $count_transactions
     */
    public function setCountTransactions(int $count_transactions): void
    {
        $this->count_transactions = $count_transactions;
    }
}
